@@ -88,12 +88,28 @@ namespace Mono.WebServer
 		bool stop;
 		bool verbose;
 		Socket listen_socket;
+		bool single_app;
 
 		Thread runner;
 
 		// This is much faster than hashtable for typical cases.
 		ArrayList vpathToHost = new ArrayList ();
 
+		public bool SingleApplication {
+			get { return single_app; }
+			set { single_app = value; }
+		}
+
+		public IApplicationHost AppHost {
+			get { return ((VPathToHost) vpathToHost [0]).AppHost; }
+			set { ((VPathToHost) vpathToHost [0]).AppHost = value; }
+		}
+
+		public IRequestBroker Broker {
+			get { return ((VPathToHost) vpathToHost [0]).RequestBroker; }
+			set { ((VPathToHost) vpathToHost [0]).RequestBroker = value; }
+		}
+
 		public int Port {
 			get {
 				if (listen_socket == null
@@ -123,8 +139,12 @@ namespace Mono.WebServer
 
 		public void AddApplication (string vhost, int vport, string vpath, string fullPath)
 		{
+			char dirSepChar = Path.DirectorySeparatorChar;
+			if (fullPath != null && !fullPath.EndsWith (dirSepChar.ToString ()))
+				fullPath += dirSepChar;
+			
 			// TODO - check for duplicates, sort, optimize, etc.
-			if (verbose) {
+			if (verbose && !single_app) {
 				Console.WriteLine("Registering application:");
 				Console.WriteLine("    Host:          {0}", (vhost != null) ? vhost : "any");
 				Console.WriteLine("    Port:          {0}", (vport != -1) ?
@@ -139,7 +159,7 @@ namespace Mono.WebServer
 
  		public void AddApplicationsFromConfigDirectory (string directoryName)
  		{
-			if (verbose) {
+			if (verbose && !single_app) {
 				Console.WriteLine ("Adding applications from *.webapp files in " +
 						   "directory '{0}'", directoryName);
 			}
@@ -156,7 +176,7 @@ namespace Mono.WebServer
 
  		public void AddApplicationsFromConfigFile (string fileName)
  		{
-			if (verbose) {
+			if (verbose && !single_app) {
 				Console.WriteLine ("Adding applications from config file '{0}'", fileName);
 			}
 
@@ -192,7 +212,7 @@ namespace Mono.WebServer
 #else
 			// TODO: support vhosts in xsp.exe
 			string name = el.SelectSingleNode ("name").InnerText;
-			if (verbose)
+			if (verbose && !single_app)
 				Console.WriteLine ("Ignoring vhost {0} for {1}", n.InnerText, name);
 #endif
 
@@ -203,7 +223,7 @@ namespace Mono.WebServer
 				vport = Convert.ToInt32 (n.InnerText);
 #else
 			// TODO: Listen on different ports
-			if (verbose)
+			if (verbose && !single_app)
 				Console.WriteLine ("Ignoring vport {0} for {1}", n.InnerText, name);
 #endif
 
@@ -218,7 +238,7 @@ namespace Mono.WebServer
  			if (applications == "")
 				return;
 
-			if (verbose) {
+			if (verbose && !single_app) {
 				Console.WriteLine("Adding applications '{0}'...", applications);
 			}
 
@@ -269,6 +289,14 @@ namespace Mono.WebServer
  			if (vpathToHost == null)
  				throw new InvalidOperationException ("SetApplications must be called first.");
 
+			if (single_app) {
+				VPathToHost v = (VPathToHost) vpathToHost [0];
+				v.AppHost = AppHost;
+				// Link the host in the application domain with a request broker in the *same* domain
+				// Not needed for SingleApplication and mod_mono
+				v.RequestBroker = webSource.CreateRequestBroker ();
+				AppHost.RequestBroker = v.RequestBroker;
+			}
 			listen_socket = webSource.CreateSocket ();
 			listen_socket.Listen (500);
 			listen_socket.Blocking = false;
@@ -381,6 +409,9 @@ namespace Mono.WebServer
 		public VPathToHost GetApplicationForPath (string vhost, int port, string path,
 							  bool defaultToRoot)
 		{
+			if (single_app)
+				return (VPathToHost) vpathToHost [0];
+
 			VPathToHost bestMatch = null;
 			int bestMatchLength = 0;
 
@@ -413,6 +444,13 @@ namespace Mono.WebServer
 			return null;
 		}
 
+		public VPathToHost GetSingleApp ()
+		{
+			if (vpathToHost.Count == 1)
+				return (VPathToHost) vpathToHost [0];
+			return null;
+		}
+
 		public void DestroyHost (IApplicationHost host)
 		{
 			// Called when the host appdomain is being unloaded
